@@ -17,7 +17,7 @@ from .run_info import RunInfo
 
 class StrategyContext(AbstractStrategyContext):
 
-    def __init__(self, kwargs: dict, timer_factory: TimerFactory, notify_order: Callable):
+    def __init__(self, kwargs: dict, timer_factory: TimerFactory):
         # 自定义定时器
         self.__tf = timer_factory
         self.__constant = Constant()  # 初始化常量
@@ -25,7 +25,7 @@ class StrategyContext(AbstractStrategyContext):
         self.__run_info = RunInfo(kwargs=kwargs, context=self)
         # 回测的当前日期，每日set_dt重置
         self.__dt = None
-        # 回测当前日期的Benchmark close price
+        # 回测当前日期的Benchmark收益率
         self.__benchmark_value = None
         self.__unit = kwargs.get('unit', '1d')  # 行情周期
         self.__parameters = kwargs.get('parameters')
@@ -35,21 +35,47 @@ class StrategyContext(AbstractStrategyContext):
         self.__stock_pool = kwargs.get('stock_pool')
 
         # 初始化Portfolio
-        self._portfolio = Portfolio(kwargs.get('init_investment', 0.0), self)
+        self._portfolio = Portfolio(self)
 
         # 初始化委托编号
         self.order_id = 0
-        self.__notify_order = notify_order  # 委托回调函数
 
         # 初始化空的当前数据字典，通过self.current_data_dict[sec_code]按需获取数据, 每日reset_current_data()被重置
         self.current_data_dict = None
 
+        # 历史委托列表, 从notify_order()函数收集委托数据，回测结束后写入数据库
+        self.orders = []
+
         """
         回测过程中画出曲线
         self.plot_data = {"000001.SZ": {"startExitPrice": {"type": "line", "xAxis": [], "yAxis": [], "desc": ""}}}
         """
         self.plot_data = dict()
 
+        """
+        记录投资组合的转入或转出资金历史
+        self.inout_cash_his = [{"datetime": self.current_dt: "cash": cash}]
+        """
+        self.inout_cash_his = []
+
+    def notify_order(self, order):
+        """
+        新创建委托的通知
+        :param order: 委托的对象
+        :return:
+        """
+        order_dict = {"order_id": order.order_id,
+                      "sec_code": order.security,
+                      "volume": order.volume,
+                      "price": order.price,
+                      "is_buy": order.is_buy,
+                      "avg_cost": order.avg_cost,
+                      "comm": order.commission,
+                      "add_time": order.add_time,
+                      "trade_date": order.add_time.strftime('%Y-%m-%d')}
+        self.orders.append(order_dict)
+        log.debug(f"order: {order_dict}")
+
     def plot_line(self, sec_code: str, name: str, value: float, desc=None):
         """
         画曲线
@@ -130,6 +156,44 @@ class StrategyContext(AbstractStrategyContext):
         """
         self.__tf.add_timer(callback=func, kwargs={"context": self}, when=time_str, weekdays=[weekday], weekcarry=True)
 
+    def run_periodically(self,
+                         func: Callable,
+                         days: int,
+                         time_str: str,
+                         start_date: str = None,
+                         end_date: str = None) -> None:
+        """
+        按调仓周期运行，如果遇到调仓日为非交易日，则顺延到下一个交易日。
+
+        :param func: 一个自定义的函数, 此函数必须接受context参数;例如自定义函数名market_open(context)
+        :param days : 间隔天数执行，start_date为第一次执行日期
+        :param time_str: 具体执行时间,一个字符串格式的时间, 24小时内的任意时间，例如"10:00", "01:00"；在实盘中才生效
+        :param start_date: 开始日期，默认为回测开始日期
+        :param end_date: 结束日期, 默认为回测结束日期
+        """
+        if start_date is None:
+            start_date = self.run_info.start_date
+        if end_date is None:
+            end_date = self.run_info.end_date
+
+        # 将字符串日期转换为datetime对象
+        start_date_ = datetime.datetime.strptime(start_date, '%Y-%m-%d')
+        end_date_ = datetime.datetime.strptime(end_date, '%Y-%m-%d')
+
+        # 初始化日期列表
+        dates = []
+        # 当当前日期小于或等于结束日期时，将其添加到列表中，并向前移动interval_days天
+        current_date = start_date_
+        while current_date <= end_date_:
+            dates.append(current_date.strftime('%Y-%m-%d'))
+            current_date += datetime.timedelta(days=days)
+
+        self.__tf.add_timer(callback=func,
+                            kwargs={"context": self},
+                            when=time_str,
+                            dates=dates,
+                            datecarry=True)
+
     def run_daily(self, func: Callable, time_str: str) -> None:
         """
         每天内何时运行
@@ -272,7 +336,8 @@ class StrategyContext(AbstractStrategyContext):
                         dividend_type=dividend_type,
                         start_date=start_date,
                         end_date=end_date,
-                        end_datetime=end_datetime)
+                        end_datetime=end_datetime,
+                        expect_df=expect_df)
         df.reset_index(inplace=True)
 
         return df
@@ -453,7 +518,7 @@ class StrategyContext(AbstractStrategyContext):
 
     def set_benchmark_value(self, value: float):
         """
-        设置回测过程中当前日期的benchmark close价格
+        设置回测过程中当前日期的benchmark的收益率
         :param value:
         :return:
         """
@@ -461,6 +526,10 @@ class StrategyContext(AbstractStrategyContext):
 
     @property
     def benchmark_value(self):
+        """
+        当前日期的benchmark close价格
+        :return:
+        """
         return self.__benchmark_value
 
     def set_dt(self, dt):
@@ -709,7 +778,7 @@ class StrategyContext(AbstractStrategyContext):
         self.order_id += 1
         order = Order(order_id=self.order_id, stock_code=stock_code, direction='buy', order_volume=volume,
                       order_price=price, avg_cost=price, context=self)
-        self.__notify_order(order)
+        self.notify_order(order)
         return order
 
     def sell(self,
@@ -759,7 +828,7 @@ class StrategyContext(AbstractStrategyContext):
             return
 
         # 持仓成本
-        open_price = position.open_price
+        avg_price = position.avg_price
 
         # 更新持仓
         update_status = position.update_position(volume=volume * -1, price=price, transact_time=self.current_dt)
@@ -776,8 +845,8 @@ class StrategyContext(AbstractStrategyContext):
         # 更新委托Id
         self.order_id += 1
         order = Order(order_id=self.order_id, stock_code=stock_code, direction='sell', order_volume=volume,
-                      order_price=price, avg_cost=open_price, context=self)
-        self.__notify_order(order)
+                      order_price=price, avg_cost=avg_price, context=self)
+        self.notify_order(order)
         return order
 
     @property
@@ -786,3 +855,14 @@ class StrategyContext(AbstractStrategyContext):
         策略投资组合，可通过该对象获取当前策略账户、持仓等信息
         """
         return self._portfolio
+
+    def inout_cash(self, cash: float):
+        """
+        投资组合转入或转出资金，当日的出入金从当日开始记入成本，用于计算收益，即当日结束计算收益时的本金是包含当日出入金金额的
+        :param cash: 可正可负，正为入金，负为出金。
+        :return:
+        """
+        # 记录出入金历史
+        self.inout_cash_his.append({"datetime": self.current_dt, "cash": cash})
+        # 把出入金更新到投资组合的可以资金
+        self.portfolio.add_cash(cash=cash)
