@@ -55,3 +55,12 @@ class AbstractPortfolio(with_metaclass(abc.ABCMeta)):
         [float] 可用资金
         """
         raise NotImplementedError
+
+    @abc.abstractmethod
+    def add_cash(self, cash: float):
+        """
+        添加/减少可用金额
+        :param cash:
+        :return:
+        """
+        raise NotImplementedError
