@@ -58,6 +58,24 @@ class AbstractStrategyContext(with_metaclass(abc.ABCMeta)):
         """
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def run_periodically(self,
+                         func: Callable,
+                         days: int,
+                         time_str: str,
+                         start_date: str = None,
+                         end_date: str = None) -> None:
+        """
+        按调仓周期运行，如果遇到调仓日为非交易日，则顺延到下一个交易日。
+
+        :param func: 一个自定义的函数, 此函数必须接受context参数;例如自定义函数名market_open(context)
+        :param days : 间隔天数执行，start_date为第一次执行日期
+        :param time_str: 具体执行时间,一个字符串格式的时间, 24小时内的任意时间，例如"10:00", "01:00"；在实盘中才生效
+        :param start_date: 开始日期，默认为回测开始日期
+        :param end_date: 结束日期, 默认为回测结束日期
+        """
+        raise NotImplementedError
+
     @abc.abstractmethod
     def run_daily(self, func: Callable, time_str: str) -> None:
         """
@@ -351,6 +369,15 @@ class AbstractStrategyContext(with_metaclass(abc.ABCMeta)):
         """
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def inout_cash(self, cash: float):
+        """
+        投资组合转入或转出资金，当日的出入金从当日开始记入成本，用于计算收益，即当日结束计算收益时的本金是包含当日出入金金额的
+        :param cash: 可正可负，正为入金，负为出金。
+        :return:
+        """
+        raise NotImplementedError
+
     @property
     @abc.abstractmethod
     def run_info(self):
